
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
// The pins for I2C are defined by the Wire-library. 
// On an arduino UNO:       A4(SDA), A5(SCL)
// On an arduino MEGA 2560: 20(SDA), 21(SCL)
// On an arduino LEONARDO:   2(SDA),  3(SCL), ...
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


//Constants:
const int relayPin = 2;  //relays control
//const int pwmPin = 3;  //pin 6 has PWM ~900Hz funtion, pin 5 has PWM ~900Hz funtion, pin 3 has PWM 488Hz funtio, mosfets control
const int stage1Pin = 3;  //pin 6 has PWM ~900Hz funtion, pin 5 has PWM ~900Hz funtion, pin 3 has PWM 488Hz funtio, mosfets control
const int stage2Pin = 6;  //pin 6 has PWM ~900Hz funtion, pin 5 has PWM ~900Hz funtion, pin 3 has PWM 488Hz funtio, mosfets control
const int rpmreadPin = 9;  //pin 6 has PWM funtion, mosfets control
const int potPin = A0; //pin A0 to read potentiometer analog input, brake voltage
const int pot2Pin = A1; //pin A1 to read potentiometer analog input, rpm brake
const int voltPin = A6; //pin A6 to read voltage analog input

//Variables:
int value; //save potentiometer analog value
int value2; //save potentiometer analog value
float volts=0.0;
float temp=0.0;
float r1=1000000.0;    //r1 value 1megom
float r2=100000.0;      //r2 value 20k gnd
int brakeV=60;
//int brakePWM=0;  //pwm 0-255
int brakeRPM=600;
int brakeRPMstatus=1;
float RPS=0;
float RPM=0;
float windSpeed=0;
int Rcount=0;
int rpmstate=0;
int stage=0;

const unsigned long measureInterval = 100; //
unsigned long previousMeasureTime = 0;

const unsigned long stage1BrakeInterval = 100; //ms stage 1
unsigned long previousStage1Time = 0;

const unsigned long stage2BrakeInterval = 150; //ms stage 2 check period has to be later than measureInterval because should have latest voltage
unsigned long previousStage2Time = 0;

const unsigned long stage3BrakeInterval = 5000; //ms stage 3 relays v1.1
unsigned long previousStage3Time = 0;

const unsigned long potsInterval = 200; //ms 
unsigned long previousPotsTime = 0;

const unsigned long displayInterval = 90; //ms 
unsigned long previousDisplayTime = 0;

const unsigned long rpsInterval = 5000; //ms for RPS rotations per secons cacluation
unsigned long previousRpsTime = 0;

const unsigned long relaysBrakeInterval = 30000; //ms for relays brake duration
unsigned long previousRelaysTime = 0;

const unsigned long rpmlowBrakeInterval = 60000; //ms for relays brake duration
unsigned long previousRpmlowTime = 0;
int rpmlow_counter=0;


void setup() {
  // put your setup code here, to run once:
 Serial.begin(9600);
 pinMode(relayPin, OUTPUT);
 pinMode(stage1Pin, OUTPUT); 
  pinMode(stage2Pin, OUTPUT);
 pinMode(rpmreadPin, INPUT); //Optional
 pinMode(potPin, INPUT); //Optional
 pinMode(pot2Pin, INPUT); //Optional
 pinMode(voltPin, INPUT); //Optional
  
   // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 OLED allocation failed"));
    //for(;;); // Don't proceed, loop forever
  }
    // Clear the buffer
  display.clearDisplay();

  brakeRPMstatus=1; //open closed relays
  digitalWrite(relayPin, HIGH); //set relays to release brake
  digitalWrite(stage1Pin, LOW);
  digitalWrite(stage2Pin, LOW);
}

void loop() {
  // put your main code here, to run repeatedly:
  unsigned long currentTime = millis();



if (currentTime -  previousPotsTime >= potsInterval) {
//read voltage brake potentionemeter
  value = analogRead(potPin);          //Read and save analog value from potentiometer
  value = map(value, 0, 1023, 0, 60); //Map value 0-1023 to 0-255 (PWM) max 60v
  brakeV=value;

//read rpm brake potentionemeter
  value2 = analogRead(pot2Pin);          //Read and save analog value from potentiometer
  value2 = map(value2, 0, 1023, 0, 600); //Map value 0-1023 to 
  brakeRPM=value2;
previousPotsTime = currentTime;
}//------------------------------


//read rpmreadPin and increase counter
if (digitalRead(rpmreadPin) == HIGH){
  if(rpmstate==1){
    Rcount=Rcount+1;
    rpmstate=0;
   //Serial.println(Rcount);
  }
}
if (digitalRead(rpmreadPin) == LOW){
rpmstate=1;  
}


//count RPS and RPM rotations per second
if (currentTime -  previousRpsTime >= rpsInterval) {
RPS=Rcount; 
RPS=RPS/2; //two pulses per rotation
RPS=RPS/5; // interval 5000ms=5sec
windSpeed=RPS*0.5969*1.18; //bbz cofic
RPM=RPS*60;
/*
Serial.print("Rcount: ");
Serial.println(Rcount);
Serial.print("RPS: ");
Serial.println(RPS);
Serial.print("RPM: ");
Serial.println(RPM);
Serial.print("Wind: ");
Serial.println(windSpeed);
*/
RPS=0;
Rcount=0; 
//https://www.electroschematics.com/anemometer/
//v=r*RPM*0.10472
//circumference
//C-apskritimo ilgis
//C=2*pi*r
//r=9.5cm / 100=0.095m
//C=2*pi*0.095
//C=0.0.5969m
//windSpeed(m/s)=RPS*0.5969
//https://www.jsdelivr.com/package/npm/node-anemometer
//factor *1.18



previousRpsTime = currentTime;
}

//call read voltage function
if (currentTime -  previousMeasureTime >= measureInterval) {
voltageRead();
previousMeasureTime = currentTime;
}
//Serial.print("volts: ");
//Serial.println(volts);

//--------rpm brake-------------------
   if(RPM>brakeRPM){ 
       if(stage==0){
       digitalWrite(stage1Pin, HIGH);
       delay(1000);
       digitalWrite(stage2Pin, HIGH);
       delay(1000);
       }
    brakeRPMstatus=0; //hit brake per relays
    digitalWrite(relayPin, LOW); //hit brake per relays
    previousRelaysTime = currentTime;
   }else{
         if (currentTime - previousRelaysTime >= relaysBrakeInterval) {
         brakeRPMstatus=1; //release relays brake
         digitalWrite(relayPin, HIGH); //release relays brake
            if(stage==0){
            digitalWrite(stage1Pin, LOW);
            digitalWrite(stage2Pin, LOW);
            }
         }
   }
//----if 5 times each minute anemometer stopped hit brakes ---
if (currentTime -  previousRpmlowTime >= rpmlowBrakeInterval) {
   if(RPM==0){ 
    rpmlow_counter=rpmlow_counter+1;
       if(rpmlow_counter==5){
       if(stage==0){
       digitalWrite(stage1Pin, HIGH);
       delay(1000);
       digitalWrite(stage2Pin, HIGH);
       delay(1000);
       }
       brakeRPMstatus=0; //hit brake per relays
       digitalWrite(relayPin, LOW); //hit brake per relays
       previousRelaysTime = currentTime;
       }
   }else{
         if (currentTime - previousRelaysTime >= relaysBrakeInterval) {
         rpmlow_counter=0;
         brakeRPMstatus=1; //release relays brake
         digitalWrite(relayPin, HIGH); //release relays brake
            if(stage==0){
            digitalWrite(stage1Pin, LOW);
            digitalWrite(stage2Pin, LOW);
            }
         }
   }
previousRpmlowTime = currentTime;
}

 //-------STAGES-----------------------------------------------
 //stage check
//--
if( volts-4>brakeV && brakeRPMstatus!=0){
if(stage==2){stage=3;
previousStage3Time=currentTime;}
}else{
if (currentTime - previousStage3Time >=  stage3BrakeInterval) {
if(stage==3) {stage=2;} 
}
}

if( volts-2 >brakeV && brakeRPMstatus!=0){
if(stage==1){stage=2;
previousStage2Time=currentTime;}
}else{
  if (currentTime - previousStage2Time >=  stage2BrakeInterval) {
  if(stage==2) {stage=1;} 
  }
}

if( volts>brakeV && brakeRPMstatus!=0){
if(stage==0){stage=1;
previousStage1Time=currentTime;}
}else{
     if (currentTime - previousStage1Time >=  stage1BrakeInterval) {
     if(stage==1) {stage=0;} 
     }
}


//-

 if(stage==0 && brakeRPMstatus!=0){
   digitalWrite(stage1Pin, LOW);
   digitalWrite(stage2Pin, LOW); 
   digitalWrite(relayPin, HIGH); //relays off
   //voltageRead();
 }
 if(stage==1 && brakeRPMstatus!=0){
 digitalWrite(stage1Pin, HIGH);
 digitalWrite(stage2Pin, LOW);
 digitalWrite(relayPin, HIGH); //relays off
 voltageRead();
 }
 if(stage==2 && brakeRPMstatus!=0){
 digitalWrite(stage1Pin, HIGH);
 digitalWrite(stage2Pin, HIGH);
 digitalWrite(relayPin, HIGH); //relays off
 voltageRead();
 }
 if(stage==3 && brakeRPMstatus!=0){
 digitalWrite(stage1Pin, HIGH);
 digitalWrite(stage2Pin, HIGH);
 digitalWrite(relayPin, LOW); //relays on
 voltageRead();
 }
 


//------------------------------------------------
  
if (currentTime -  previousDisplayTime >= displayInterval) {  
 display.clearDisplay();
   //display.setFont(&FreeSerif9pt7b);
   //display.setFont(FreeSans24pt7b);
   //tft.setFont(&FreeMonoBoldOblique12pt7b);
   display.setTextSize(1,2);      // Normal 1:1 pixel scale
   display.setTextColor(SSD1306_WHITE); // Draw white text
      display.setCursor(0,0);
      display.println("Volts: ");
      display.setCursor(40,0); // 8symbols x5=40
      display.println(volts,1);
    display.setCursor(70,0);
    display.println("RPM: ");    
    display.setCursor(95,0);
    display.println(RPM,1);  
    
   display.setCursor(0,16);
   display.println("BrakeV: ");
   display.setCursor(45,16); // 9symbols x5=45
   display.println(brakeV);
    display.setCursor(65,16);
    display.println("Brpm");  
    display.setCursor(95,16);
    display.println(brakeRPM);

   display.setCursor(56,32);
   display.print("Wind: ");
   display.print(windSpeed,1);
   display.println("m/s");

   if(stage>0){
      display.setCursor(0,50);
      display.print("*Stage ");
      display.print(stage);
      display.println("*"); 
   }
   if (brakeRPMstatus==0){
     display.setCursor(65,50);
     display.println("*RPM Stop*");
   }
       display.display();
   //delay(200);
previousDisplayTime=currentTime;
}//end of display 

  
} //end of loop


void voltageRead(){
  volts=0;
  float analogvalue=0;
  for(int i=0; i<10; i++){
      analogvalue += analogRead(voltPin);
  }
     analogvalue=analogvalue/10;
      temp = (analogvalue * 5.1) / 1024.0;  // FORMULA USED TO CONVERT THE VOLTAGE
    volts = temp / (r2/(r1+r2));
    //volts=volts+0.6;//correction
    if (volts < 0.9)  { volts=0.0;} 
}
